/* $Id: driver_test.c,v 1.4 2005/04/30 14:24:45 dijkstra Exp $ */

/*
 * Copyright (c) 2004-2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifdef TEST
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "driver.h"
#include "error.h"
#include "lcd.h"

char *driver_name = "Test";
int driver_capabilities = DC_BLOCK | DC_HBAR | DC_VBAR;
int driver_x = 0, driver_y = 0;

void
driver_connect(char *connect, int capabilities)
{
    debug("initialize %s driver @ %s", driver_name, connect);
}
char
driver_get_hchar(int percentage)
{
    percentage %= 101;
    return (percentage / 10) + '0';
}
char
driver_get_vchar(int percentage)
{
    percentage %= 101;
    return (percentage / 10) + '0';
}
void
driver_nprintf(int maxsize, char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size;

    maxsize %= sizeof(buf);

    va_start(args, fmt);
    size = vsnprintf(buf, maxsize, fmt, args);
    va_end(args);

    info("%02d,%02d:%s", driver_x, driver_y, buf);
}
void
driver_printf(char *fmt, ...)
{
    va_list args;
    char buf[LCD_MAXSTR];
    int size;

    va_start(args, fmt);
    size = vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    info("%02d,%02d:%s", driver_x, driver_y, buf);
}
void
driver_reset()
{
    info("driver:reset");
}
void
driver_set_backlight(int backlight)
{
    info("driver:set backlight");
}
void
driver_set_contrast(int contrast)
{
    info("driver:set contrast");
}
void
driver_set_specialchar(int ch, struct specialchar *data)
{
    info("driver:set specialchar");
}
void
driver_set_xy(int x, int y)
{
    driver_x = x;
    driver_y = y;
}
#endif /* DRIVER_TEST */
