/* $Id: port.c,v 1.2 2005/04/30 14:24:45 dijkstra Exp $ */

/*
 * Copyright (c) 2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#if defined LPT_GLIBC
#include <sys/io.h>
#elif defined LPT_BSD
#include <sys/types.h>
#include <machine/pio.h>
#include <machine/sysarch.h>
#elif defined LPT_FREEBSD
#include <sys/types.h>
#include <machine/cpufunc.h>
#include <machine/sysarch.h>
#endif

#include "error.h"

#if defined LPT_GLIBC || defined LPT_BSD || defined LPT_FREEBSD
int
readport(unsigned short int port)
{
    return inb(port);
}

void
writeport(unsigned short int port, unsigned char val) {
#if defined LPT_GLIBC
    outb(val, port);
#else
    outb(port, val);
#endif
}


#if defined LPT_GLIBC
void
accessport (unsigned short int port)
{
    if (ioperm(port, 1, 255) == -1) {
	fatal("cannot get io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
}
#elif defined LPT_BSD
void
accessport(unsigned short int port)
{
    unsigned long iomap[32];

    if (i386_get_ioperm(iomap) == -1) {
	fatal("cannot get io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }

    iomap[port >> 5] &= ~(1 << (port & 0x1f));

    if (i386_set_ioperm(iomap) == -1) {
	fatal("cannot set io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
}
#elif defined LPT_FREEBSD
void
accessport(unsigned short int port)
{
    if (i386_set_ioperm(port, 1, 1) == -1) {
	fatal("cannot get io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
}
#endif
#endif
